// Type definitions for WebDriver v5
// Project: https://www.npmjs.com/package/webdriver
// Definitions by: auto generated by https://github.com/webdriverio/webdriverio
// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped
/// <reference types="node"/>

declare type HTTPRequestOptions = import('got').Options;
declare type HTTPResponse = import('got').Response;

declare namespace WebDriver {
    type PageLoadingStrategy = 'none' | 'eager' | 'normal';
    type ProxyTypes = 'pac' | 'noproxy' | 'autodetect' | 'system' | 'manual';
    type WebDriverLogTypes = 'trace' | 'debug' | 'info' | 'warn' | 'error' | 'silent';
    type LoggingPreferenceType =
        'OFF' | 'SEVERE' | 'WARNING' |
        'INFO' | 'CONFIG' | 'FINE' |
        'FINER' | 'FINEST' | 'ALL';
    type FirefoxLogLevels =
        'trace' | 'debug' | 'config' |
        'info' | 'warn' | 'error' | 'fatal';
    type Timeouts = 'script' | 'pageLoad' | 'implicit';
    export type SameSiteOptions = 'Lax' | 'Strict';

    interface ProxyObject {
        proxyType?: ProxyTypes;
        proxyAutoconfigUrl?: string;
        ftpProxy?: string;
        ftpProxyPort?: number;
        httpProxy?: string;
        httpProxyPort?: number;
        sslProxy?: string;
        sslProxyPort?: number;
        socksProxy?: string;
        socksProxyPort?: number;
        socksVersion?: string;
        socksUsername?: string;
        socksPassword?: string;
    }

    interface LoggingPreferences {
        browser?: LoggingPreferenceType;
        driver?: LoggingPreferenceType;
        server?: LoggingPreferenceType;
        client?: LoggingPreferenceType;
    }

    interface Cookie {
        /**
         * The name of the cookie.
         */
        name: string;
        /**
         * The cookie value.
         */
        value: string;
        /**
         * The cookie path. Defaults to "/" if omitted when adding a cookie.
         */
        path?: string;
        /**
         * The domain the cookie is visible to. Defaults to the current browsing context’s
         * active document’s URL domain if omitted when adding a cookie.
         */
        domain?: string;
        /**
         * 	Whether the cookie is a secure cookie. Defaults to false if omitted when adding
         * a cookie.
         */
        secure?: boolean;
        /**
         * Whether the cookie is an HTTP only cookie. Defaults to false if omitted when
         * adding a cookie.
         */
        httpOnly?: boolean;
        /**
         * When the cookie expires, specified in seconds since Unix Epoch. Must not be set if
         * omitted when adding a cookie.
         */
        expiry?: number;
        /**
         * Whether the cookie applies to a SameSite policy. Defaults to None if omitted when
         * adding a cookie. Can be set to either "Lax" or "Strict".
         */
        sameSite?: SameSiteOptions
    }

    interface ChromeOptions {
        /**
         * List of command-line arguments to use when starting Chrome. Arguments with an
         * associated value should be separated by a '=' sign (e.g., `['start-maximized', 'user-data-dir=/tmp/temp_profile']`).
         * See here for a list of Chrome arguments.
         */
        args?: string[];
        /**
         * Path to the Chrome executable to use (on Mac OS X, this should be the actual binary,
         * not just the app. e.g., '/Applications/Google Chrome.app/Contents/MacOS/Google Chrome')
         */
        binary?: string;
        /**
         * A list of Chrome extensions to install on startup. Each item in the list should
         * be a base-64 encoded packed Chrome extension (.crx)
         */
        extensions?: string[];
        /**
         * A dictionary with each entry consisting of the name of the preference and its value.
         * These preferences are applied to the Local State file in the user data folder.
         */
        localState?: {
            [name: string]: any;
        };
        /**
         * If false, Chrome will be quit when ChromeDriver is killed, regardless of whether
         * the session is quit. If true, Chrome will only be quit if the session is quit
         * (or closed). Note, if true, and the session is not quit, ChromeDriver cannot clean
         * up the temporary user data directory that the running Chrome instance is using.
         */
        detach?: boolean;
        /**
         * An address of a Chrome debugger server to connect to, in the form of `<hostname/ip:port>`,
         * e.g. '127.0.0.1:38947'
         */
        debuggerAddress?: string;
        /**
         * List of Chrome command line switches to exclude that ChromeDriver by default passes
         * when starting Chrome.  Do not prefix switches with --.
         */
        excludeSwitches?: string[];
        /**
         * Directory to store Chrome minidumps . (Supported only on Linux.)
         */
        minidumpPath?: string;
        /**
         * 	A dictionary with either a value for “deviceName,” or values for “deviceMetrics” and
         * “userAgent.” Refer to Mobile Emulation for more information.
         */
        mobileEmulation?: {
            [name: string]: any;
        };
        /**
         * An optional dictionary that specifies performance logging preferences. See
         * [Chromedriver docs](http://chromedriver.chromium.org/capabilities) for
         * more information.
         */
        perfLoggingPrefs?: {
            enableNetwork?: boolean;
            enablePage?: boolean;
            enableTimeline?: boolean;
            tracingCategories?: boolean;
            bufferUsageReportingInterval?: boolean;
        };
        /**
         * A dictionary with each entry consisting of the name of the preference and its value.
         * These preferences are only applied to the user profile in use. See the 'Preferences'
         * file in Chrome's user data directory for examples.
         */
        prefs?: {
            [name: string]: string | number | boolean;
        };
        /**
         * A list of window types that will appear in the list of window handles. For access
         * to <webview> elements, include "webview" in this list.
         */
        windowTypes?: string[];
    }

    interface FirefoxLogObject {
        level: FirefoxLogLevels
    }

    interface FirefoxOptions {
        binary?: string,
        args?: string[],
        profile?: string,
        log?: FirefoxLogObject,
        prefs?: {
            [name: string]: string | number | boolean;
        }
    }

    interface Capabilities {
        /**
         * Identifies the user agent.
         */
        browserName?: string;
        /**
         * 	Identifies the version of the user agent.
         */
        browserVersion?: string;
        /**
         * Identifies the operating system of the endpoint node.
         */
        platformName?: string;
        /**
         * Indicates whether untrusted and self-signed TLS certificates are implicitly trusted on navigation for the duration of the session.
         */
        acceptInsecureCerts?: boolean;
        /**
         * Defines the current session’s page load strategy.
         */
        pageLoadStrategy?: PageLoadingStrategy;
        /**
         * Defines the current session’s proxy configuration.
         */
        proxy?: ProxyObject;
        /**
         * Indicates whether the remote end supports all of the resizing and repositioning commands.
         */
        setWindowRect?: boolean;
        /**
         * Describes the timeouts imposed on certain session operations.
         */
        timeouts?: Timeouts;
        /**
         * Defines the current session’s strict file interactability.
         */
        strictFileInteractability?: boolean,
        /**
         * Describes the current session’s user prompt handler. Defaults to the dismiss and notify state.
         */
        unhandledPromptBehavior?: string;
    }

    interface DesiredCapabilities extends Capabilities {
        // Read-only capabilities
        cssSelectorsEnabled?: boolean;
        handlesAlerts?: boolean;
        version?: string;
        platform?: string;

        // Read-write capabilities
        javascriptEnabled?: boolean;
        databaseEnabled?: boolean;
        locationContextEnabled?: boolean;
        applicationCacheEnabled?: boolean;
        browserConnectionEnabled?: boolean;
        webStorageEnabled?: boolean;
        acceptSslCerts?: boolean;
        rotatable?: boolean;
        nativeEvents?: boolean;
        unexpectedAlertBehaviour?: string;
        elementScrollBehavior?: number;

        // Grid-specific
        seleniumProtocol?: string;
        maxInstances?: number;
        environment?: string;

        // Selenium RC (1.0) only
        commandLineFlags?: string;
        executablePath?: string;
        timeoutInSeconds?: number;
        onlyProxySeleniumTraffic?: boolean;
        avoidProxy?: boolean;
        proxyEverything?: boolean;
        proxyRequired?: boolean;
        browserSideLog?: boolean;
        optionsSet?: boolean;
        singleWindow?: boolean;
        dontInjectRegex?: RegExp;
        userJSInjection?: boolean;
        userExtensions?: string;

        // RemoteWebDriver specific
        'webdriver.remote.sessionid'?: string;
        'webdriver.remote.quietExceptions'?: boolean;

        // Selenese-Backed-WebDriver specific
        'selenium.server.url'?: string;

        loggingPrefs?: {
            browser?: LoggingPreferences;
            driver?: LoggingPreferences;
            server?: LoggingPreferences;
            client?: LoggingPreferences;
        };

        // Firefox
        firefox_binary?: string;
        firefoxProfileTemplate?: string;
        captureNetworkTraffic?: boolean;
        addCustomRequestHeaders?: boolean;
        trustAllSSLCertificates?: boolean;
        changeMaxConnections?: boolean;
        profile?: string;
        pageLoadingStrategy?: string;
        'moz:firefoxOptions'?: FirefoxOptions;

        // Edge specific
        'ms:inPrivate'?: boolean;
        'ms:extensionPaths'?: string[];
        'ms:startPage'?: string;

        // IE specific
        'ie.forceCreateProcessApi'?: boolean;
        'ie.browserCommandLineSwitches'?: string;
        'ie.usePerProcessProxy'?: boolean;
        'ie.ensureCleanSession'?: boolean;
        'ie.setProxyByServer'?: boolean;
        'ie.fileUploadDialogTimeout'?: number;
        'ie.edgechromium'?: boolean;
        'ie.edgepath'?: string;
        ignoreProtectedModeSettings?: boolean;
        ignoreZoomSetting?: boolean;
        initialBrowserUrl?: string;
        enablePersistentHover?: boolean;
        enableElementCacheCleanup?: boolean;
        requireWindowFocus?: boolean;
        browserAttachTimeout?: number;
        logFile?: string;
        logLevel?: string;
        host?: string;
        extractPath?: string;
        silent?: string;
        killProcessesByName?: boolean;

        // Safari specific
        'safari.options'?: {
            [name: string]: any;
        };

        // Chrome specific
        chromeOptions?: ChromeOptions;
        'goog:chromeOptions'?: ChromeOptions;
        mobileEmulationEnabled?: boolean;

        // webdriverio specific
        specs?: string[];
        exclude?: string[];

        // wdio-sauce-service specific
        build?: string;

        // Saucelabs w3c specific
        'sauce:options'?: {
            [name: string]: any;
        };

        // Browserstack w3c specific
        'bstack:options'?: {
            [name: string]: any;
        };

        // Testingbot w3c specific
        'tb:options'?: {
            [name: string]: any;
        };

        // Appium General Capabilities
        automationName?: string;
        platformVersion?: string;
        deviceName?: string;
        app?: string;
        newCommandTimeout?: number;
        language?: string;
        locale?: string;
        udid?: string;
        orientation?: string;
        autoWebview?: boolean;
        noReset?: boolean;
        fullReset?: boolean;
        eventTimings?: boolean;
        enablePerformanceLogging?: boolean;
        printPageSourceOnFindFailure?: boolean;

        // Appium Android Only
        appActivity?: string;
        appPackage?: string;
        appWaitActivity?: string;
        appWaitPackage?: string;
        appWaitDuration?: number;
        deviceReadyTimeout?: number;
        allowTestPackages?: boolean;
        androidCoverage?: string;
        androidCoverageEndIntent?: string;
        androidDeviceReadyTimeout?: number;
        androidInstallTimeout?: number;
        androidInstallPath?: string;
        adbPort?: number;
        systemPort?: number;
        remoteAdbHost?: string;
        androidDeviceSocket?: string;
        avd?: string;
        avdLaunchTimeout?: number;
        avdReadyTimeout?: number;
        avdArgs?: string;
        useKeystore?: boolean;
        keystorePath?: string;
        keystorePassword?: string;
        keyAlias?: string;
        keyPassword?: string;
        chromedriverExecutable?: string;
        chromedriverArgs?: string[];
        chromedriverExecutableDir?: string;
        chromedriverChromeMappingFile?: string;
        chromedriverUseSystemExecutable?: boolean;
        autoWebviewTimeout?: number;
        chromedriverPort?: number;
        chromedriverPorts?: (number | number[])[]
        intentAction?: string;
        intentCategory?: string;
        intentFlags?: string;
        optionalIntentArguments?: string;
        dontStopAppOnReset?: boolean;
        unicodeKeyboard?: boolean;
        resetKeyboard?: boolean;
        noSign?: boolean;
        ignoreUnimportantViews?: boolean;
        disableAndroidWatchers?: boolean;
        recreateChromeDriverSessions?: boolean;
        nativeWebScreenshot?: boolean;
        androidScreenshotPath?: string;
        autoGrantPermissions?: boolean;
        networkSpeed?: string;
        gpsEnabled?: boolean;
        isHeadless?: boolean;
        adbExecTimeout?: number;
        localeScript?: string;
        skipDeviceInitialization?: boolean;
        chromedriverDisableBuildCheck?: boolean;
        skipUnlock?: boolean;
        unlockType?: string;
        unlockKey?: string;
        autoLaunch?: boolean;
        skipLogcatCapture?: boolean;
        uninstallOtherPackages?: string;
        disableWindowAnimation?: boolean;
        otherApps?: string;
        uiautomator2ServerLaunchTimeout?: number;
        uiautomator2ServerInstallTimeout?: number;
        skipServerInstallation?: boolean;
        espressoServerLaunchTimeout?: number;

        // Appium iOS Only
        calendarFormat?: string;
        bundleId?: string;
        launchTimeout?: number;
        locationServicesEnabled?: boolean;
        locationServicesAuthorized?: boolean;
        autoAcceptAlerts?: boolean;
        autoDismissAlerts?: boolean;
        nativeInstrumentsLib?: boolean;
        nativeWebTap?: boolean;
        safariInitialUrl?: string;
        safariAllowPopups?: boolean;
        safariIgnoreFraudWarning?: boolean;
        safariOpenLinksInBackground?: boolean;
        keepKeyChains?: boolean;
        localizableStringsDir?: string;
        processArguments?: string;
        interKeyDelay?: number;
        showIOSLog?: boolean;
        sendKeyStrategy?: string;
        screenshotWaitTimeout?: number;
        waitForAppScript?: string;
        webviewConnectRetries?: number;
        appName?: string;
        customSSLCert?: string;
        webkitResponseTimeout?: number;
        remoteDebugProxy?: string;
        enableAsyncExecuteFromHttps?: boolean;
        skipLogCapture?: boolean;
        webkitDebugProxyPort?: number;
    }

    interface Options {
        /**
         * Protocol to use when communicating with the Selenium standalone server (or driver).
         */
        protocol?: string;
        /**
         * Host of your WebDriver server.
         */
        hostname?: string;
        /**
         * Port your WebDriver server is on.
         */
        port?: number;
        /**
         * Path to WebDriver endpoint or grid server.
         */
        path?: string;
        /**
         * Query paramaters that are propagated to the driver server.
         */
        queryParams?: {
            [name: string]: string;
        },
        /**
         * Defines the [capabilities](https://w3c.github.io/webdriver/webdriver-spec.html#capabilities) you want to run in your Selenium session.
         */
        capabilities?: DesiredCapabilities;
        requestedCapabilities?: DesiredCapabilities;
        /**
         * Level of logging verbosity.
         */
        logLevel?: WebDriverLogTypes;
        /**
         * Set specific log levels per logger
         * use 'silent' level to disable logger
         */
        logLevels?: object;
        /**
         * Timeout for any WebDriver request to a driver or grid.
         */
        connectionRetryTimeout?: number;
        /**
         * Count of request retries to the Selenium server.
         */
        connectionRetryCount?: number;
        /**
         * Specify custom headers to pass into every request.
         */
        headers?: {
            [name: string]: string;
        };
        /**
         * Function intercepting [HTTP request options](https://github.com/sindresorhus/got#options) before a WebDriver request is made.
         */
        transformRequest?: (requestOptions: HTTPRequestOptions) => HTTPRequestOptions;
        /**
         * Function intercepting HTTP response objects after a WebDriver response has arrived.
         */
        transformResponse?: (response: HTTPResponse, requestOptions: HTTPRequestOptions) => HTTPResponse;
    }

    interface AttachSessionOptions extends Options {
        sessionId: string,
        isW3C?: boolean
    }

    function newSession(
        options?: Options,
        modifier?: (...args: any[]) => any,
        proto?: object,
        commandWrapper?: (commandName: string, fn: (...args: any[]) => any) => any
    ): Promise<Client>;

    function attachToSession(
        options: AttachSessionOptions,
        modifier?: (...args: any[]) => any,
        proto?: object,
        commandWrapper?: (commandName: string, fn: (...args: any[]) => any) => any
    ): Promise<Client>;

    function reloadSession(
        instance: Client
    ): Promise<Client>;

    interface ClientOptions {
        capabilities: DesiredCapabilities;
        isW3C: boolean;
        isChrome: boolean;
        isAndroid: boolean;
        isMobile: boolean;
        isIOS: boolean;
        sessionId: string;
    }

    // object with no match
    interface ProtocolCommandResponse {
        [key: string]: any;
    }

    // webdriver.json
    interface SessionReturn extends DesiredCapabilities, ProtocolCommandResponse { }

    interface StatusReturn extends ProtocolCommandResponse {
        ready?: boolean,
        message?: string,
    }

    interface WindowHandle {
        handle: string,
        type: string
    }

    interface RectReturn {
        x: number,
        y: number,
        width: number,
        height: number
    }

    // appium.json
    interface StringsReturn {
        [key:string]: string
    }

    interface SettingsReturn extends ProtocolCommandResponse {
        shouldUseCompactResponses?: boolean,
        elementResponseAttributes?: string,
        ignoreUnimportantViews?: boolean,
        allowInvisibleElements?: boolean,
        enableNotificationListener?: boolean,
        actionAcknowledgmentTimeout?: number,
        keyInjectionDelay?: number,
        scrollAcknowledgmentTimeout?: number,
        waitForIdleTimeout?: number,
        waitForSelectorTimeout?: number,
        normalizeTagNames?: boolean,
        shutdownOnPowerDisconnect?: boolean,
        mjpegServerScreenshotQuality?: number,
        mjpegServerFramerate?: number,
        screenshotQuality?: number,
        mjpegScalingFactor?: number,
    }

    interface BaseClient {
        // id of WebDriver session
        sessionId: string;
        // assigned capabilities by the browser driver / WebDriver server
        capabilities: DesiredCapabilities;
        // original requested capabilities
        requestedCapabilities: DesiredCapabilities;

        /**
         * browser flags
         */
        // true if session runs on a mobile device
        isMobile: boolean;
        // true if mobile session runs on iOS
        isIOS: boolean;
        // true if mobile session runs on Android
        isAndroid: boolean;
    }

    // generated typings
    // ... insert here ...

    interface ClientAsync extends AsyncClient, BaseClient { }
}

type AsyncClient = {
    [K in keyof Pick<WebDriver.Client, Exclude<keyof WebDriver.Client, keyof WebDriver.BaseClient>>]:
    (...args: Parameters<WebDriver.Client[K]>) => Promise<ReturnType<WebDriver.Client[K]>>;
}

declare module "webdriver" {
    export = WebDriver;
}
